//////////////////////////////////////////////////////////////////////////
// Name:	FMODAudio.h
// Date:	January 27th, 2014
// Author:	Tylor Emmett [tsemmett@gmail.com] for GSP 381
// Purpose: This file is used to create a very simple framework for the
//			FMOD Audio system.
//////////////////////////////////////////////////////////////////////////

//FMOD Sound System, copyright  Firelight Technologies Pty, Ltd., 1994-2013.


#pragma once

#include "fmod.hpp"
#include "fmod_errors.h"
#pragma comment (lib, "Fmodex_vc.lib")


#define BOUNCE_SOUND	"Audio/BounceFX.mp3"
#define SCROLL_SOUND	"Audio/ScrollFX.mp3"
#define SELECT_SOUND	"Audio/SelectFX.mp3"
#define GOAL_SOUND		"Audio/GoalFX.mp3"
#define MENU_SONG		"Audio/Pinball Spring.mp3"
#define GAME_SONG		"Audio/Show Your Moves.mp3"


class FMODAudio
{
	//////////////////////////////////////////////////////////////////////////
	// FMOD Variables
	//////////////////////////////////////////////////////////////////////////
	FMOD::System	*m_pSystem;				//Main audio system
	FMOD::Channel	*m_pChannelFX;			//Channel for sound fx
	FMOD::Channel	*m_pChannelBkg;			//Channel for menu background music
	FMOD::Sound		*m_pBounceFX;			//Bounce sound effect
	FMOD::Sound		*m_pMenuScrollFX;		//Scolling through menu sound effect
	FMOD::Sound		*m_pMenuSelectFX;		//Selecting an object in menu effect
	FMOD::Sound		*m_pGoalFX;				//Sound played when a goal is scored.
	FMOD::Sound		*m_pMenuBkg;			//Background music for menu
	FMOD::Sound		*m_pGameBkg;			//Background music for game
	FMOD_RESULT		m_Result;				//FMOD Result for error checking
	FMOD::Sound		*m_pCurrentSound;		//For testing the current sound on the background channel.
	bool			m_isGameBkgPlaying;		
	bool			m_isMenuBkgPlaying;

	//////////////////////////////////////////////////////////////////////////
	// Name:		CheckError
	// Parameters:	FMOD_RESULT - error to be checked
	// Return:		void
	// Description:	Processes FMOD error codes
	//////////////////////////////////////////////////////////////////////////
	void CheckError(FMOD_RESULT);			//For reading an error (ERRCHK didn't seem to work for me).
	
public:	
	//////////////////////////////////////////////////////////////////////////
	// Init and Shutdown are preferred to constructors and destructor,
	// due to having more control when to explicitly call them when global.
	//////////////////////////////////////////////////////////////////////////
	FMODAudio();
	~FMODAudio();

	//////////////////////////////////////////////////////////////////////////
	// Name:		Init
	// Parameters:	HWND hWnd - Handle to the window for the application
	//				HINSTANCE hInst - Handle to the application instance
	//				bool bWindowed - Boolean to control windowed or full-screen
	// Return:		void
	// Description:	Ran once at the start.  Initialize DirectX components and 
	//				variables to control the application.  
	//////////////////////////////////////////////////////////////////////////
	void Init();

	//////////////////////////////////////////////////////////////////////////
	// Name:		Update
	// Parameters:	none
	// Return:		void
	// Description:	Run each frame. Updates sound system
	//////////////////////////////////////////////////////////////////////////
	void Update();
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		Shutdown
	// Parameters:	none
	// Return:		void
	// Description:	Ran once at game end. Releases sound system.
	//////////////////////////////////////////////////////////////////////////
	void Shutdown();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlayBounceFX
	// Parameters:	none
	// Return:		void
	// Description:	Plays the bounce sound effect 
	//////////////////////////////////////////////////////////////////////////
	void PlayBounceFX();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlayScrollFX
	// Parameters:	none
	// Return:		void
	// Description:	Plays the scroll sound effect 
	//////////////////////////////////////////////////////////////////////////
	void PlayScrollFX();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlaySelectFX
	// Parameters:	none
	// Return:		void
	// Description:	Plays the scroll sound effect 
	//////////////////////////////////////////////////////////////////////////
	void PlaySelectFX();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlayGoalFX
	// Parameters:	none
	// Return:		void
	// Description:	Plays the scroll sound effect 
	//////////////////////////////////////////////////////////////////////////
	void PlayGoalFX();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlayMenuBkg
	// Parameters:	none
	// Return:		void
	// Description:	Starts the Menu Background track if it was not already running
	//				Stops game background track first if running.
	//////////////////////////////////////////////////////////////////////////
	void PlayMenuBkg();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		PlayGameBkg
	// Parameters:	none
	// Return:		void
	// Description:	Starts the Game Background track if it was not already running
	//				Stops menu background track first if running.
	//////////////////////////////////////////////////////////////////////////
	void PlayGameBkg();
private:
	/////////////////////////////////////////////////////////////////////////
	// Name:		IsGameBkgPlaying
	// Parameters:	none
	// Return:		bool - is the game background music playing?
	// Description:	Tests for the game background track. Used when switching tracks
	//////////////////////////////////////////////////////////////////////////
	bool IsGameBkgPlaying();
	
	/////////////////////////////////////////////////////////////////////////
	// Name:		IsMenuBkgPlaying
	// Parameters:	none
	// Return:		bool - is the menu background music playing?
	// Description:	Tests for the menu background track. Used when switching tracks
	//////////////////////////////////////////////////////////////////////////
	bool IsMenuBkgPlaying();
		
};

