//////////////////////////////////////////////////////////////////////////
// Name:	GameObjects.h
// Date:	January 27th, 2014
// Author:	Tylor Emmett [tsemmett@gmail.com]
// Purpose: This file creates 2 objects for use in a Pong-style game:
//			a paddle, and a ball. It also provides methods
//			for processing collisions between these objects.
//////////////////////////////////////////////////////////////////////////
#pragma once
#include <d3d9.h>
#include <d3dx9.h>
#include "Enums.h"
#include <ctime>
#include "DirectInput.h"
#include "FMODAudio.h"

extern CDirectInputFramework	g_Input;		//Link to global Input object
extern FMODAudio				g_Audio;		//Link to global Audio object

////////////////////////////////////////////////////////////////
// Class Prototypes
////////////////////////////////////////////////////////////////
class Paddle;
class Ball;

////////////////////////////////////////////////////////////////
//	Ball Class
//
//	Provides a ball with Pong-like behavior, and a sphere
//	collision object. Supports collision between a paddle or
//	wall object.
////////////////////////////////////////////////////////////////
class Ball
{

private:
	//////////////////////////////////////////////////////
	// Member variables
	//////////////////////////////////////////////////////	
	float					m_Radius;			//Ball's collision radius
	D3DXVECTOR3				m_Position;			//Ball's Position
	D3DXVECTOR3				m_Velocity;			//Ball's Velocity
	D3DXVECTOR3				m_StartPos;			//Ball's start position
	RECT					m_Window;			//Window size
	float					m_ElapsedTime;		//Time accumulator for startup

public:
	/////////////////////////////////////////////////////
	//	Default Constructors - Use Init and Shutdown to
	//	start and stop this object.
	//////////////////////////////////////////////////////
	Ball();
	~Ball();
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		Init
	// Parameters:	D3DXVECTOR3 location - starting location for ball
	//				D3DXIMAGE_INFO* imageInfo - image info struct to setup collision
	// Return:		none
	// Description:	Prepares object for play. Loads textures and initializes
	//				variables.
	//////////////////////////////////////////////////////////////////////////
	void Init(D3DXVECTOR3& location, D3DXIMAGE_INFO* imageInfo, RECT window);

	//////////////////////////////////////////////////////////////////////////
	// Name:		Reset
	// Parameters:	none
	// Return:		none
	// Description:	Reset's ball to center position. Used after scoring a goal
	//////////////////////////////////////////////////////////////////////////
	void Reset();

	//////////////////////////////////////////////////////////////////////////
	// Name:		Update
	// Parameters:	float dt - Change in time between frames
	// Return:		none
	// Description:	Updates object's location based on time and input
	//////////////////////////////////////////////////////////////////////////
	void Update(float dt);

	//////////////////////////////////////////////////////////////////////////
	// Name:		CheckContact
	// Parameters:	Ball& ball - Pointer to the ball object being checked
	//				Paddle& paddle - Pointer to the paddle being checked
	// Return:		none
	// Description:	Checks for contact between a ball and paddle. Friend
	//				method between the ball and paddle classes.
	//////////////////////////////////////////////////////////////////////////
	friend void CheckContactPaddle(Ball* ball, Paddle* paddle);

	//////////////////////////////////////////////////////////////////////////
	// Name:		CheckContact
	// Parameters:	none
	//				
	// Return:		Player - player scored
	// Description:	Checks for contact between a ball and wall. 
	//				If contacted left or right, returns the player who scored
	//////////////////////////////////////////////////////////////////////////
	Player CheckContactWall();

	//////////////////////////////////////////////////////////////////////////
	// Name:		Reflect
	// Parameters:	Axis - Determines which axis to reflect about
	// Return:		void
	// Description:	Reflects the ball about the axis passed.
	//////////////////////////////////////////////////////////////////////////
	void Reflect(Axis axis);
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		getPosition
	// Parameters:	none
	// Return:		D3DXVECTOR3 - current position
	// Description:	Returns ball's current position.
	//////////////////////////////////////////////////////////////////////////
	D3DXVECTOR3 getPosition();
};

////////////////////////////////////////////////////////////////
//	Paddle Class
//
//	Provides a paddle with Pong-like behavior, and a box
//	collision object. Supports collision between a ball object.
////////////////////////////////////////////////////////////////
class Paddle
{

private:
public:
	//////////////////////////////////////////////////////
	// Member variables
	//////////////////////////////////////////////////////
	float				m_contactHalfX;				//X Half Size for contact and position update
	float				m_contactHalfY;				//Y Half Size for contact and position update.
	D3DXVECTOR3			m_Position;					//Position of the object
	D3DXVECTOR3			m_StartPos;					//Start Position
	float				m_Velocity;					//Object's velocity (only moves along y axis, so float is sufficient
	float				m_MaxVelocity;				//Maximum velocity
	Player				m_Player;					//Player number
	RECT				m_Window;					//Window coords
	
public:
	/////////////////////////////////////////////////////
	//	Default Constructors - Use Init and Shutdown to
	//	start and stop this object.
	//////////////////////////////////////////////////////
	Paddle();
	~Paddle();
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		Init
	// Parameters:	D3DXVECTOR3 location - Starting location for paddle.
	//				Player player - Player number.
	//				D3DXIMAGE_INFO* imageInfo
	//				RECT window - window size for collision
	// Return:		none
	// Description:	Prepares object for play. Loads textures and initializes
	//				variables.
	//////////////////////////////////////////////////////////////////////////
	void Init(D3DXVECTOR3& location, D3DXIMAGE_INFO* imageInfo, Player player, RECT window);
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		Reset
	// Parameters:	none
	// Return:		none
	// Description:	Reset's paddle to starting position. Used after scoring a goal
	//////////////////////////////////////////////////////////////////////////
	void Reset();

	//////////////////////////////////////////////////////////////////////////
	// Name:		Update
	// Parameters:	float dt - Change in time between frames
	// Return:		none
	// Description:	Updates object's location based on time and input
	//////////////////////////////////////////////////////////////////////////
	void Update(float dt);

	//////////////////////////////////////////////////////////////////////////
	// Name:		getPlayer
	// Parameters:	none
	// Return:		Player
	// Description:	Returns player (to determine which sprite to draw)
	//////////////////////////////////////////////////////////////////////////
	Player getPlayer();
	
	//////////////////////////////////////////////////////////////////////////
	// Name:		getPosition
	// Parameters:	none
	// Return:		D3DXVECTOR3 - current position
	// Description:	Returns the paddle's current position.
	//////////////////////////////////////////////////////////////////////////
	D3DXVECTOR3 getPosition();

	//////////////////////////////////////////////////////////////////////////
	// Name:		CheckContact
	// Parameters:	Ball* ball - Pointer to the ball object being checked
	//				Paddle* paddle - Pointer to the paddle being checked
	// Return:		none
	// Description:	Checks for contact between a ball and paddle. Friend
	//				method between the ball and paddle classes.
	//////////////////////////////////////////////////////////////////////////
	friend void CheckContact(Ball* ball, Paddle* paddle);
};



